@@ -13,10 +13,10 @@ from livestreamer.compat import urlparse, urljoin
 from livestreamer.exceptions import NoStreamsError, PluginError, StreamError
 from livestreamer.options import Options
 from livestreamer.plugin import Plugin
-from livestreamer.plugin.api import http
+from livestreamer.plugin.api import http, validate
+from livestreamer.plugin.api.utils import parse_json, parse_query
 from livestreamer.stream import (HTTPStream, HLSStream, FLVPlaylist,
                                  extract_flv_header_tags)
-from livestreamer.utils import parse_json, parse_qsd, verifyjson
 
 __all__ = ["PluginBase", "APIBase"]
 
@@ -27,10 +27,61 @@ QUALITY_WEIGHTS = {
     "low": 240,
     "mobile": 120,
 }
-URL_PATTERN = (r"http(s)?://([\w\.]+)?(?P<domain>twitch.tv|justin.tv)/(?P<channel>\w+)"
-               r"(/(?P<video_type>[bc])/(?P<video_id>\d+))?")
 USHER_SELECT_PATH = "/select/{0}.json"
 
+_url_re = re.compile(r"""
+    http(s)?://
+    (?:
+        (?P<subdomain>\w+)
+        \.
+    )?
+    (?P<domain>twitch.tv|justin.tv)
+    /
+    (?P<channel>[^/]+)
+    (?:
+        /
+        (?P<video_type>[bc])
+        /
+        (?P<video_id>\d+)
+    )?
+""", re.VERBOSE)
+
+_access_token_schema = validate.Schema(
+    {
+        "token": validate.text,
+        "sig": validate.text
+    },
+    validate.union((
+        validate.get("sig"),
+        validate.get("token")
+    ))
+)
+_token_schema = validate.Schema(
+    {
+        "chansub": {
+            "restricted_bitrates": validate.all(
+                [validate.text],
+                validate.filter(
+                    lambda n: not re.match(r"(.+_)?archives|live|chunked", n)
+                )
+            )
+        }
+    },
+    validate.get("chansub")
+)
+_viewer_info_schema = validate.Schema(
+    {
+        validate.optional("login"): validate.text
+    },
+    validate.get("login")
+)
+_viewer_token_schema = validate.Schema(
+    {
+        validate.optional("token"): validate.text
+    },
+    validate.get("token")
+)
+
 
 class UsherService(object):
     def __init__(self, host="justin.tv"):
@@ -42,9 +93,15 @@ class UsherService(object):
 
     def select(self, channel, password=None, **extra_params):
         url = self.url(USHER_SELECT_PATH, channel)
-        params = dict(p=int(random() * 999999), type="any",
-                      allow_source="true", private_code=password or "null",
-                      **extra_params)
+        params = {
+            "private_code": password or "null",
+            "player": "twitchweb",
+            "p": int(random() * 999999),
+            "type": "any",
+            "allow_source": "true",
+            "allow_audio_only": "true",
+        }
+        params.update(extra_params)
 
         req = requests.Request("GET", url, params=params)
         # prepare_request is only available in requests 2.0+
@@ -57,44 +114,42 @@ class UsherService(object):
 
 
 class APIBase(object):
-    def __init__(self, host="justin.tv"):
+    def __init__(self, host="justin.tv", beta=False):
+        self.beta = beta
         self.host = host
         self.oauth_token = None
+        self.subdomain = beta and "betaapi" or "api"
 
     def add_cookies(self, cookies):
         http.parse_cookies(cookies, domain=self.host)
 
-    def call(self, path, format="json", host=None, **extra_params):
+    def call(self, path, format="json", host=None, schema=None, **extra_params):
         params = dict(as3="t", **extra_params)
 
         if self.oauth_token:
             params["oauth_token"] = self.oauth_token
 
-        url = "https://api.{0}{1}.{2}".format(host or self.host, path, format)
-        # The certificate used by Twitch cannot be verified in some
-        # environments.
+        url = "https://{0}.{1}{2}.{3}".format(self.subdomain, host or self.host,
+                                              path, format)
+        # The certificate used by Twitch cannot be verified in some environments.
         res = http.get(url, params=params, verify=False)
 
         if format == "json":
-            return http.json(res)
+            return http.json(res, schema=schema)
         elif format == "xml":
-            return http.xml(res)
+            return http.xml(res, schema=schema)
         else:
             return res
 
-    def channel_access_token(self, channel):
-        res = self.call("/api/channels/{0}/access_token".format(channel),
-                        host="twitch.tv")
+    def channel_access_token(self, channel, **params):
+        return self.call("/api/channels/{0}/access_token".format(channel),
+                         host="twitch.tv", **params)
 
-        return res.get("sig"), res.get("token")
+    def token(self, **params):
+        return self.call("/api/viewer/token", host="twitch.tv", **params)
 
-    def token(self):
-        res = self.call("/api/viewer/token", host="twitch.tv")
-
-        return res.get("token")
-
-    def viewer_info(self):
-        return self.call("/api/viewer/info", host="twitch.tv")
+    def viewer_info(self, **params):
+        return self.call("/api/viewer/info", host="twitch.tv", **params)
 
 
 class PluginBase(Plugin):
@@ -115,17 +170,19 @@ class PluginBase(Plugin):
         Plugin.__init__(self, url)
 
         try:
-            match = re.match(URL_PATTERN, url).groupdict()
+            match = _url_re.match(url).groupdict()
             self.channel = match.get("channel").lower()
+            self.subdomain = match.get("subdomain")
             self.video_type = match.get("video_type")
             self.video_id = match.get("video_id")
             self.usher = UsherService(match.get("domain"))
 
             parsed = urlparse(url)
-            self.params = parse_qsd(parsed.query)
+            self.params = parse_query(parsed.query)
         except AttributeError:
             self.channel = None
             self.params = None
+            self.subdomain = None
             self.video_id = None
             self.video_type = None
             self.usher = None
@@ -244,10 +301,8 @@ class PluginBase(Plugin):
             self.logger.info("Attempting to authenticate using cookies")
 
             self.api.add_cookies(cookies)
-            self.api.oauth_token = self.api.token()
-
-            viewer = self.api.viewer_info()
-            login = viewer.get("login")
+            self.api.oauth_token = self.api.token(schema=_viewer_token_schema)
+            login = self.api.viewer_info(schema=_viewer_info_schema)
 
             if login:
                 self.logger.info("Successfully logged in as {0}", login)
@@ -257,7 +312,9 @@ class PluginBase(Plugin):
 
     def _access_token(self):
         try:
-            sig, token = self.api.channel_access_token(self.channel)
+            sig, token = self.api.channel_access_token(
+                self.channel, schema=_access_token_schema
+            )
         except PluginError as err:
             if "404 Client Error" in str(err):
                 raise NoStreamsError(self.url)
@@ -284,15 +341,12 @@ class PluginBase(Plugin):
                 raise PluginError(err)
 
         try:
-            token = parse_json(token)
-            chansub = verifyjson(token, "chansub")
-            restricted_bitrates = verifyjson(chansub, "restricted_bitrates")
-
-            for name in filter(lambda n: not re.match(r"(.+_)?archives|live", n),
-                               restricted_bitrates):
-                self.logger.warning("The quality '{0}' is not available "
-                                    "since it requires a subscription.",
-                                    name)
+            token = parse_json(token, schema=_token_schema)
+            for name in token["restricted_bitrates"]:
+                if name not in streams:
+                    self.logger.warning("The quality '{0}' is not available "
+                                        "since it requires a subscription.",
+                                        name)
         except PluginError:
             pass
 
