@@ -1,6 +1,5 @@
 import errno
 import os
-import re
 import requests
 import sys
 import signal
@@ -18,11 +17,12 @@ from livestreamer.stream import StreamProcess
 from .argparser import parser
 from .compat import stdout, is_win32
 from .console import ConsoleOutput
-from .constants import CONFIG_FILE, PLUGINS_DIR, STREAM_SYNONYMS
+from .constants import CONFIG_FILES, PLUGINS_DIR, STREAM_SYNONYMS
 from .output import FileOutput, PlayerOutput
 from .utils import NamedPipe, HTTPServer, ignored, stream_to_url
 
 ACCEPTABLE_ERRNO = (errno.EPIPE, errno.EINVAL, errno.ECONNRESET)
+QUIET_OPTIONS = ("json", "stream_url", "subprocess_cmdline", "quiet")
 
 args = console = livestreamer = plugin = None
 
@@ -151,8 +151,7 @@ def output_stream_http(plugin, streams):
                     stream = None
 
             except PluginError as err:
-                console.logger.error("Unable to fetch new streams: {0}",
-                                     err)
+                console.logger.error(u"Unable to fetch new streams: {0}", err)
 
             if not stream:
                 console.logger.info("Stream not available, will re-fetch "
@@ -162,7 +161,8 @@ def output_stream_http(plugin, streams):
                 continue
 
             try:
-                console.logger.info("Opening stream: {0}", stream_name)
+                console.logger.info("Opening stream: {0} ({1})", stream_name,
+                                    type(stream).shortname())
                 stream_fd, prebuffer = open_stream(stream)
             except StreamError as err:
                 console.logger.error("{0}", err)
@@ -226,10 +226,13 @@ def open_stream(stream):
 def output_stream(stream):
     """Open stream, create output and finally write the stream to output."""
 
-    try:
-        stream_fd, prebuffer = open_stream(stream)
-    except StreamError as err:
-        console.logger.error("{0}", err)
+    for i in range(args.retry_open):
+        try:
+            stream_fd, prebuffer = open_stream(stream)
+            break
+        except StreamError as err:
+            console.logger.error("{0}", err)
+    else:
         return
 
     output = create_output()
@@ -323,7 +326,7 @@ def handle_stream(plugin, streams, stream_name):
 
     # Print internal command-line if this stream
     # uses a subprocess.
-    if args.cmdline:
+    if args.subprocess_cmdline:
         if isinstance(stream, StreamProcess):
             try:
                 cmdline = stream.cmdline()
@@ -332,15 +335,18 @@ def handle_stream(plugin, streams, stream_name):
 
             console.msg("{0}", cmdline)
         else:
-            console.exit("Stream does not use a command-line")
+            console.exit("The stream specified cannot be translated to a command")
 
     # Print JSON representation of the stream
     elif console.json:
         console.msg_json(stream)
 
-    # Continuously output the stream over HTTP
-    elif args.player_continuous_http and not (args.output or args.stdout):
-        output_stream_http(plugin, streams)
+    elif args.stream_url:
+        url = stream_to_url(stream)
+        if url:
+            console.msg("{0}", url)
+        else:
+            console.exit("The stream specified cannot be translated to a URL")
 
     # Output the stream
     else:
@@ -348,16 +354,21 @@ def handle_stream(plugin, streams, stream_name):
         # to use these in case the main stream is not usable.
         alt_streams = list(filter(lambda k: stream_name + "_alt" in k,
                                   sorted(streams.keys())))
+        file_output = args.output or args.stdout
 
         for stream_name in [stream_name] + alt_streams:
-            console.logger.info("Opening stream: {0}", stream_name)
             stream = streams[stream_name]
             stream_type = type(stream).shortname()
 
-            if (stream_type in args.player_passthrough and
-                not (args.output or args.stdout)):
+            if stream_type in args.player_passthrough and not file_output:
+                console.logger.info("Opening stream: {0} ({1})", stream_name,
+                                    stream_type)
                 success = output_stream_passthrough(stream)
+            elif args.player_continuous_http and not file_output:
+                return output_stream_http(plugin, streams)
             else:
+                console.logger.info("Opening stream: {0} ({1})", stream_name,
+                                    stream_type)
                 success = output_stream(stream)
 
             if success:
@@ -371,6 +382,29 @@ def fetch_streams(plugin):
                               sorting_excludes=args.stream_sorting_excludes)
 
 
+def fetch_streams_infinite(plugin, interval):
+    """Attempts to fetch streams until some are returned."""
+
+    try:
+        streams = fetch_streams(plugin)
+    except PluginError as err:
+        console.logger.error(u"{0}", err)
+        streams = None
+
+    if not streams:
+        console.logger.info("Waiting for streams, retrying every {0} "
+                            "second(s)", args.retry_streams)
+    while not streams:
+        sleep(args.retry_streams)
+
+        try:
+            streams = fetch_streams(plugin)
+        except PluginError as err:
+            console.logger.error(u"{0}", err)
+
+    return streams
+
+
 def resolve_stream_name(streams, stream_name):
     """Returns the real stream name of a synonym."""
 
@@ -424,18 +458,30 @@ def handle_url():
         plugin = livestreamer.resolve_url(args.url)
         console.logger.info("Found matching plugin {0} for URL {1}",
                             plugin.module, args.url)
-        streams = fetch_streams(plugin)
+
+        if args.retry_streams:
+            streams = fetch_streams_infinite(plugin, args.retry_streams)
+        else:
+            streams = fetch_streams(plugin)
     except NoPluginError:
         console.exit("No plugin can handle URL: {0}", args.url)
     except PluginError as err:
-        console.exit("{0}", err)
+        console.exit(u"{0}", err)
 
     if not streams:
         console.exit("No streams found on this URL: {0}", args.url)
 
+    if args.best_stream_default:
+        args.default_stream = ["best"]
+
+    if args.default_stream and not args.stream and not args.json:
+        args.stream = args.default_stream
+
     if args.stream:
+        validstreams = format_valid_streams(streams)
         for stream_name in args.stream:
             if stream_name in streams:
+                console.logger.info("Available streams: {0}", validstreams)
                 handle_stream(plugin, streams, stream_name)
                 return
 
@@ -446,7 +492,6 @@ def handle_url():
             console.msg_json(dict(streams=streams, plugin=plugin.module,
                                   error=err))
         else:
-            validstreams = format_valid_streams(streams)
             console.exit("{0}.\n       Available streams: {1}",
                          err, validstreams)
     else:
@@ -503,15 +548,14 @@ def load_plugins(dirs):
                                    "a directory!", directory)
 
 
-def setup_args():
+def setup_args(config_files=[]):
     """Parses arguments."""
     global args
-
     arglist = sys.argv[1:]
 
-    # Load additional arguments from livestreamerrc
-    if os.path.exists(CONFIG_FILE):
-        arglist.insert(0, "@" + CONFIG_FILE)
+    # Load arguments from config files
+    for config_file in filter(os.path.isfile, config_files):
+        arglist.insert(0, "@" + config_file)
 
     args = parser.parse_args(arglist)
 
@@ -520,6 +564,27 @@ def setup_args():
         args.stream = [stream.lower() for stream in args.stream]
 
 
+def setup_config_args():
+    config_files = []
+
+    if args.url:
+        with ignored(NoPluginError):
+            plugin = livestreamer.resolve_url(args.url)
+            config_files += ["{0}.{1}".format(fn, plugin.module) for fn in CONFIG_FILES]
+
+    if args.config:
+        # We want the config specified last to get highest priority
+        config_files += list(reversed(args.config))
+    else:
+        # Only load first available default config
+        for config_file in filter(os.path.isfile, CONFIG_FILES):
+            config_files.append(config_file)
+            break
+
+    if config_files:
+        setup_args(config_files)
+
+
 def setup_console():
     """Console setup."""
     global console
@@ -533,7 +598,7 @@ def setup_console():
         console.set_output(sys.stderr)
 
     # We don't want log output when we are printing JSON or a command-line.
-    if not (args.json or args.cmdline or args.quiet):
+    if not any(getattr(args, attr) for attr in QUIET_OPTIONS):
         console.set_level(args.loglevel)
 
     if args.quiet_player:
@@ -541,24 +606,48 @@ def setup_console():
                                "version 1.4.3 as hiding player output is now "
                                "the default.")
 
+    if args.best_stream_default:
+        console.logger.warning("The option --best-stream-default is deprecated "
+                               "since version 1.9.0, use '--default-stream best' "
+                               "instead.")
+
     console.json = args.json
 
     # Handle SIGTERM just like SIGINT
     signal.signal(signal.SIGTERM, signal.default_int_handler)
 
 
-def setup_proxies():
-    """Sets the HTTP(S) proxies for this process."""
+def setup_http_session():
+    """Sets the global HTTP settings, such as proxy and headers."""
     if args.http_proxy:
-        if not re.match("^http(s)?://", args.http_proxy):
-            args.http_proxy = "http://" + args.http_proxy
-        os.environ["http_proxy"] = args.http_proxy
+        livestreamer.set_option("http-proxy", args.http_proxy)
 
     if args.https_proxy:
-        if not re.match("^http(s)?://", args.https_proxy):
-            args.https_proxy = "https://" + args.https_proxy
-        os.environ["https_proxy"] = args.https_proxy
+        livestreamer.set_option("https-proxy", args.https_proxy)
 
+    if args.http_cookies:
+        livestreamer.set_option("http-cookies", args.http_cookies)
+
+    if args.http_headers:
+        livestreamer.set_option("http-headers", args.http_headers)
+
+    if args.http_query_params:
+        livestreamer.set_option("http-query-params", args.http_query_params)
+
+    if args.http_ignore_env:
+        livestreamer.set_option("http-trust-env", False)
+
+    if args.http_no_ssl_verify:
+        livestreamer.set_option("http-ssl-verify", False)
+
+    if args.http_ssl_cert:
+        livestreamer.set_option("http-ssl-cert", args.http_ssl_cert)
+
+    if args.http_ssl_cert_crt_key:
+        livestreamer.set_option("http-ssl-cert", tuple(args.http_ssl_cert_crt_key))
+
+    if args.http_timeout:
+        livestreamer.set_option("http-timeout", args.http_timeout)
 
 def setup_plugins():
     """Loads any additional plugins."""
@@ -578,25 +667,46 @@ def setup_livestreamer():
 
 def setup_options():
     """Sets Livestreamer options."""
+    if args.hls_live_edge:
+        livestreamer.set_option("hls-live-edge", args.hls_live_edge)
 
-    livestreamer.set_option("errorlog", args.errorlog)
+    if args.hls_segment_attempts:
+        livestreamer.set_option("hls-segment-attempts", args.hls_segment_attempts)
 
-    if args.rtmpdump:
-        livestreamer.set_option("rtmpdump", args.rtmpdump)
+    if args.hls_segment_timeout:
+        livestreamer.set_option("hls-segment-timeout", args.hls_segment_timeout)
 
-    if args.rtmpdump_proxy:
-        livestreamer.set_option("rtmpdump-proxy", args.rtmpdump_proxy)
+    if args.hls_timeout:
+        livestreamer.set_option("hls-timeout", args.hls_timeout)
 
-    if args.hds_live_edge is not None:
+    if args.hds_live_edge:
         livestreamer.set_option("hds-live-edge", args.hds_live_edge)
 
-    if args.hds_fragment_buffer is not None:
-        livestreamer.set_option("hds-fragment-buffer",
-                                args.hds_fragment_buffer)
+    if args.http_stream_timeout:
+        livestreamer.set_option("http-stream-timeout", args.http_stream_timeout)
 
     if args.ringbuffer_size:
         livestreamer.set_option("ringbuffer-size", args.ringbuffer_size)
 
+    if args.rtmp_proxy:
+        livestreamer.set_option("rtmp-proxy", args.rtmp_proxy)
+
+    if args.rtmp_rtmpdump:
+        livestreamer.set_option("rtmp-rtmpdump", args.rtmp_rtmpdump)
+
+    if args.rtmp_timeout:
+        livestreamer.set_option("rtmp-timeout", args.rtmp_timeout)
+
+    livestreamer.set_option("subprocess-errorlog", args.subprocess_errorlog)
+
+    # Deprecated options
+    if args.hds_fragment_buffer:
+        console.logger.warning("The option --hds-fragment-buffer is deprecated "
+                               "and will be removed in the future. Use "
+                               "--ringbuffer-size instead")
+
+def setup_plugin_options():
+    """Sets Livestreamer plugin options."""
     if args.jtv_cookie:
         livestreamer.set_plugin_option("justintv", "cookie",
                                        args.jtv_cookie)
@@ -633,6 +743,14 @@ def setup_options():
         livestreamer.set_plugin_option("crunchyroll", "purge_credentials",
                                        args.crunchyroll_purge_credentials)
 
+    if args.livestation_email:
+        livestreamer.set_plugin_option("livestation", "email",
+                                       args.livestation_email)
+
+    if args.livestation_password:
+        livestreamer.set_plugin_option("livestation", "password",
+                                       args.livestation_password)
+
     # Deprecated options
     if args.jtv_legacy_names:
         console.logger.warning("The option --jtv/twitch-legacy-names is "
@@ -670,30 +788,38 @@ def check_version():
         latest_version = data.get("info").get("version")
         cache.set("latest_version", latest_version, (60 * 60 * 24))
 
+    version_info_printed = cache.get("version_info_printed")
+    if version_info_printed:
+        return
+
     installed_version = StrictVersion(livestreamer.version)
     latest_version = StrictVersion(latest_version)
 
     if latest_version > installed_version:
         console.logger.info("A new version of Livestreamer ({0}) is "
                             "available!".format(latest_version))
+        cache.set("version_info_printed", True, (60 * 60 * 6))
 
 
 def main():
     setup_args()
     check_root()
     setup_livestreamer()
-    setup_console()
-    setup_proxies()
     setup_plugins()
+    setup_config_args()
+    setup_console()
+    setup_http_session()
 
-    with ignored(Exception):
-        check_version()
+    if not args.no_version_check:
+        with ignored(Exception):
+            check_version()
 
     if args.plugins:
         print_plugins()
     elif args.url:
         with ignored(KeyboardInterrupt):
             setup_options()
+            setup_plugin_options()
             handle_url()
     elif args.twitch_oauth_authenticate:
         authenticate_twitch_oauth()
